# -*- coding: utf-8 -*-
# Moovida - Home multimedia server
# Copyright (C) 2006-2009 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Moovida with Fluendo's plugins.
#
# The GPL part of Moovida is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Moovida" in the root directory of this distribution package
# for details on that license.
#
# Author: Florian Boucault <florian@fluendo.com>

"""
Widget combining a title, a text entry and a keyboard.
"""

from elisa.plugins.pigment.graph.image import Image
from elisa.plugins.pigment.graph.text import Text
from elisa.plugins.pigment.widgets.box import VBox, HBox
from elisa.plugins.pigment.widgets.widget import Widget
from elisa.plugins.pigment.widgets.button import Button
from elisa.plugins.pigment.widgets.input import TextEntry as StockTextEntry
from elisa.plugins.pigment.widgets.keyboard import Keyboard as StockKeyboard
from elisa.plugins.pigment.widgets.theme import Theme
from elisa.plugins.pigment.widgets.panel import PiecePanel
from elisa.plugins.pigment.widgets.const import STATE_NORMAL, \
                                                STATE_PRESSED, \
                                                STATE_SELECTED
from elisa.plugins.pigment.animation import implicit

from elisa.plugins.poblesec.widgets.background import WidgetWithBackground
from elisa.plugins.poblesec.widgets.button import RelookSmoothStatesWidget, RelookSmoothPanel

import gobject


class TextEntry(StockTextEntry):
    """
    DOCME
    """

    def _create_widgets(self):
        super(TextEntry, self)._create_widgets()

        state_widgets = {STATE_NORMAL: TextEntryDecorations,
                         STATE_PRESSED: TextEntryDecorations,
                         STATE_SELECTED: TextEntryDecorations}

        self.decorations = RelookSmoothStatesWidget(state_widgets=state_widgets)
        self.decorations.visible = True
        self.add(self.decorations)

    def do_state_changed(self, previous_state):
        super(TextEntry, self).do_state_changed(previous_state)
        self.decorations.state = self.state


class TextEntryDecorations(Widget):

    def __init__(self):
        super(TextEntryDecorations, self).__init__()
        self._create_widgets()
        self.update_style_properties(self.style.get_items())

    def _create_widgets(self):
        self.background = PiecePanel()
        self.background.visible = True
        self.add(self.background)

        self.glyph = Image()
        self.glyph.visible = True
        self.add(self.glyph)

        self.box = PiecePanel()
        self.box.set_name("text_box")
        self.box.visible = True
        self.add(self.box)

    def do_state_changed(self, previous_state):
        super(TextEntryDecorations, self).do_state_changed(previous_state)
        self.background.state = self.state
        self.box.state = self.state


class KeyboardButton(Button):
    """
    DOCME
    """
    def do_state_changed(self, previous_state):
        self.panel.state = self.state
        super(KeyboardButton, self).do_state_changed(previous_state)

    def _get_background_style(self):
        return self.panel.name

    def _set_background_style(self, value):
        if not hasattr(self, 'panel'):
            self.panel = RelookSmoothPanel()
            self.panel.visible = True
            self.add(self.panel, forward_signals=False)
        self.panel.set_name(value)

    background_style = property(_get_background_style, _set_background_style)


class ButtonWithGlyph(KeyboardButton):
    pass


class ButtonWithoutGlyph(KeyboardButton):
    pass


class KeyboardList(gobject.GObject):

    """
    An ordered list of keyboard widgets, each associated to a given layout.

    Emit a switched-keyboard signal when the keyboard has been changed.
    """

    __gsignals__ = \
        {'switched-keyboard': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())}

    def __init__(self, layouts):
        """
        @param layouts: a list of layout resource identifiers
        @type layouts:  C{list} of C{str}
        """
        super(KeyboardList, self).__init__()

        if not layouts:
            raise ValueError('At least one layout is required.')

        self.keyboards = []

        for layout in layouts:
            theme = Theme.get_default()
            xml_layout = theme.get_resource(layout)
            keyboard = StockKeyboard(xml_layout, ButtonWithGlyph,
                                     ButtonWithoutGlyph)
            keyboard.connect('key-press-special', self._key_press_special_cb)
            self.keyboards.append(keyboard)

        self._current_index = 0

    def clean(self):
        for keyboard in self.keyboards:
            keyboard.disconnect_by_func(self._key_press_special_cb)
            keyboard.clean()

    def next(self):
        """
        Switch to the next keyboard in the list (the list is cyclic, when its
        last keyboard is reached it goes back to the first one).
        """
        prev_index = self._current_index
        self._current_index = (self._current_index + 1 ) % len(self.keyboards)
        if self._current_index != prev_index:
            self.emit('switched-keyboard')

    def _key_press_special_cb(self, keyboard, value):
        if value == 'switch_keyboard':
            self.next()


class KeyboardsWidget(Widget):

    """
    A widget that contains a list of keyboard widgets and shows only one at a
    given time.
    """

    def __init__(self, keyboards):
        """
        param keyboards: a list of keyboard widgets
        type keyboards:  C{list} of
                         L{elisa.plugins.pigment.widgets.keyboard.Keyboard}
        """
        super(KeyboardsWidget, self).__init__()

        if not keyboards:
            raise ValueError('At least one keyboard is required.')

        self.keyboards = keyboards
        self._current_index = 0

        for keyboard in keyboards:
            keyboard.opacity = 0
            self.add(keyboard)

        # Show the first keyboard.
        keyboard = self.keyboards[self._current_index]
        keyboard.visible = True
        keyboard.opacity = 255
        self.set_focus_proxy(keyboard)

    def _show_keyboard(self, index):
        keyboard = self.keyboards[index]
        self.set_focus_proxy(keyboard)
        keyboard.animated.setup_next_animations(duration=140,
                                             transformation=implicit.DECELERATE)
        keyboard.visible = True
        keyboard.animated.opacity = 255

    def _hide_keyboard(self, index):
        keyboard = self.keyboards[index]
        keyboard.animated.setup_next_animations(duration=250,
                                             transformation=implicit.DECELERATE)
        keyboard.animated.opacity = 0

    def next(self):
        """
        Hide the current keyboard and show the next one.
        """
        self._hide_keyboard(self._current_index)
        self._current_index = (self._current_index + 1) % len(self.keyboards)
        self._show_keyboard(self._current_index)


class Keyboard(VBox):

    """
    DOCME
    """

    def __init__(self, layouts=['elisa.plugins.poblesec.osk_qwerty']):
        self.layouts = layouts
        super(Keyboard, self).__init__()

    def _create_keyboard(self):
        keyboard_box = HBox()

        self.left_spacer = Image()
        self.left_spacer.visible = True
        keyboard_box.pack_start(self.left_spacer, expand=True)

        self.keyboard_list = KeyboardList(self.layouts)
        self.keyboards = KeyboardsWidget(self.keyboard_list.keyboards)
        self.keyboard_list.connect('switched-keyboard',
                                   self._switch_keyboard_cb)
        self.keyboards.visible = True
        keyboard_box.pack_start(self.keyboards)
        keyboard_box.set_focus_proxy(self.keyboards)

        self.right_spacer = Image()
        self.right_spacer.visible = True
        keyboard_box.pack_start(self.right_spacer, expand=True)
        return keyboard_box

    def create_widgets(self):
        self.navigable = True

        self.title = WidgetWithBackground(foreground=Text())
        self.title.visible = True
        self.pack_start(self.title)

        self.entry = TextEntry()
        self.entry.visible = True
        self.pack_start(self.entry)

        self.keyboard_layout = self._create_keyboard()
        self.keyboard_layout.visible = True
        self.pack_start(self.keyboard_layout)
        self.set_focus_proxy(self.keyboard_layout)

        for keyboard in self.keyboards.keyboards:
            keyboard.connect('key-press-char', self._key_press_char_cb)
            keyboard.connect('key-press-special', self._key_press_special_cb)

    def clean(self):
        self.keyboard_list.disconnect_by_func(self._switch_keyboard_cb)
        for keyboard in self.keyboards.keyboards:
            keyboard.disconnect_by_func(self._key_press_special_cb)
            keyboard.disconnect_by_func(self._key_press_char_cb)
        self.keyboard_list.clean()
        return super(Keyboard, self).clean()

    def _key_press_char_cb(self, keyboard, char):
        self.entry.content += char

    def _key_press_special_cb(self, keyboard, value):
        if value == 'delete':
            self.entry.content = self.entry.content[:-1]
        elif value == 'enter':
            self.entry.emit('activated')

    def _switch_keyboard_cb(self, keyboard_list):
        focus = self.keyboards.focus
        self.keyboards.next()
        if focus:
            # Grab the focus back if the keyboard has it previously.
            self.keyboards.set_focus()

    @classmethod
    def _demo_widget(cls, *args, **kwargs):
        widget = cls()
        widget.title.foreground.label = "TYPE SOMETHING IN"
        widget.size = (400.0, 250.0)
        return widget


if __name__ == '__main__':
    from elisa.plugins.poblesec.widgets.keyboard import Keyboard
    Keyboard.demo()
    try:
        __IPYTHON__
    except NameError:
        import pgm
        pgm.main()
